@@ -44,14 +44,39 @@ def mappend(g, c):
 
 def _in(d):
     ret = []
+    neg = False
     for i in d:
         if i[0] == 'range':
-            ret.extend(map(chr, range(i[1][0], i[1][1]+1)))
+            subs = map(chr, range(i[1][0], i[1][1]+1))
+            if neg:
+                for char in subs:
+                    try:
+                        ret.remove(char)
+                    except:
+                        pass
+            else:
+                ret.extend(subs)
         elif i[0] == 'literal':
-            ret.append(chr(i[1]))
+            if neg:
+                try:
+                    ret.remove(chr(i[1]))
+                except:
+                    pass
+            else:
+                ret.append(chr(i[1]))
         elif i[0] == 'category':
             subs = CATEGORIES.get(i[1], [''])
-            ret.extend(subs)
+            if neg:
+                for char in subs:
+                    try:
+                        ret.remove(char)
+                    except:
+                        pass
+            else:
+                ret.extend(subs)
+        elif i[0] == 'negate':
+            ret = list(CATEGORIES['category_any'])
+            neg = True
     return ret
 
 
@@ -107,6 +132,12 @@ def _gen(d, limit=20, count=False):
         # ignore ^ and $
         elif i[0] == 'at':
             continue
+        elif i[0] == 'not_literal':
+            subs = list(CATEGORIES['category_any'])
+            subs.remove(chr(i[1]))
+            if count:
+                strings = (strings or 1) * len(subs)
+            ret = comb(ret, subs)
         else:
             print('[!] cannot handle expression "%r"' % i)
 
@@ -142,6 +173,10 @@ def _randone(d, limit=20):
             ret += choice(list(chain.from_iterable(_gen(list(x[1]), limit) for x in l)))
         elif i[0] == 'at':
             continue
+        elif i[0] == 'not_literal':
+            c=list(CATEGORIES['category_any'])
+            c.remove(chr(i[1]))
+            ret += choice(c)
         else:
             print('[!] cannot handle expression "%s"' % str(i))
 
